local CG = LibStub("AceAddon-3.0"):NewAddon("CoolGlow", "AceConsole-3.0", "AceEvent-3.0", "AceHook-3.0", "AceTimer-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("CoolGlow")
local LDBI = LibStub("LibDBIcon-1.0")
local LCG = LibStub('LibCustomGlow-1.0')

CG.version = GetAddOnMetadata("CoolGlow", "Version")
CG.addonName = "|cfff2f251Cool Glow|r"
CG.icon = [[Interface\Addons\CoolGlow\media\icon.tga]]

function CG:Print(...)
	print("|cfff2f251".."Cool Glow|r:", ...)
end

---- [Defaults & Options] ----

local defaults = {
	char = {
		Welcome = true,
		Notify = true,
		minimap = {
			hide = false,
		},
		Style = "Pixel",
		Color = {r = 0.95, g = 0.95, b = 0.32, a = 1},
		Button = {
			Frequency = 0.2,
		},
		Pixel = {
			NumLines = 8,
			Frequency = 0.45,
			Length = 8,
			Thickness = 2,
			XOffset = 0,
			YOffset = 0
		},
		AutoCast = {
			NumParticles = 8,
			Frequency = 0.45,
			Scale = 1,
			XOffset = 0,
			YOffset = 0
		},
	}
}

local options = {
	type = "group",
	handler = CG,
	name = L["Cool Glow"],
	childGroups = "tab",
	args = {
		intro = {
			order = 1,
			type = 'description',
			fontSize = "medium",
			name = L["CG_DESC"],
			image = function() return CG.icon, 150, 150 end,
		},
		Space1 = {
			order = 2,
			type = "description",
			name = "",
		},
		discordlink = {
			order = 3,
			type = "input",
			width = "full",
			name = L["Join me on Discord!"],
			get = function(info) return L["https://discord.gg/GbQbDRX"] end,
		},
		Space2 = {
			order = 4,
			type = "description",
			name = "",
		},
		Welcome = {
			order = 5,
			name = L["Welcome Message"],
			desc = L["Show the welcome message when logging in."],
			type = "toggle",
			get = function(info) return CG.db.char.Welcome end,
			set = function(info, value) CG.db.char.Welcome = value end, 			
		},
		Notify = {
			order = 6,
			type = "toggle",
			name = L["New Version Notify"],
			desc = L["Show the notify message when a new version has been released."],
			get = function(info) return CG.db.char.Notify end,
			set = function(info, value) CG.db.char.Notify = value end, 
		},
		minimap = {
			order = 7,
			type = "toggle",
			name = L["Minimap Button"],
			desc = L["Show the minimap button."],
			get = function() return not CG.db.char.minimap.hide end,
			set = function(info, value)
				CG.db.char.minimap.hide = not value
				if value then
						LDBI:Show("CoolGlow")
					else
						LDBI:Hide("CoolGlow")
					end
				end,
		},
		Space3 = {
			order = 8,
			type = "description",
			name = "",
		},
		Reset = {
			order = 9,
			type = "execute",
			name = L["Reset Settings"],
			desc = L["Reset the settings of this addon to their defaults."],
			func = function()
				CoolGlowDB = defaults
				CG:Print(L["The settings were reset to the default state."])
				StaticPopup_Show("COOLGLOW_RLUI")
			end,
		},
		Test = {
			order = 10,
			type = "execute",
			name = L["Display Glow"],
			desc = L["TEST_DESC"],
			func = function() CG:ToggleTestFrame() end,
		},
		Space4 = {
			order = 11,
			type = "description",
			name = "",
		},
		Style = {
			order = 12,
			type = "select",
			name = L["Style"],
			get = function(info) return CG.db.char.Style end,
			set = function(info, value) CG.db.char.Style = value end,
			values = {
				["Button"] = L["Button Glow"],
				["Pixel"] = L["Pixel Glow"],
				["AutoCast"] = L["AutoCast Glow"],
			},
		},
		Color = {
			order = 13,
			type = "color",
			name = L["Color"],
			hasAlpha = true,
			get = function(info)
				local t = CG.db.char.Color
				return t.r, t.g, t.b, t.a
			end,
			set = function(info, r, g, b, a)
				local t = CG.db.char.Color
				t.r, t.g, t.b, t.a = r, g, b, a
				CG:ActionbarGlow()
			end,
		},
		Button = {
			order = 20,
			type = "group",
			name = L["Button Glow"],
			get = function(info) return CG.db.char.Button[ info[#info] ] end,
			set = function(info, value) CG.db.char.Button[ info[#info] ] = value; CG:ActionbarGlow() end,
			args = {
				Frequency = {
					order = 1,
					type = "range",
					name = L["Frequency"],
					desc = L["Animation speed. Negative values will rotate anti-clockwise."],
					min = 0, max = 1, step = 0.1,
				},
			},
		},
		Pixel = {
			type = "group",
			name = L["Pixel Glow"],
			order = 21,
			get = function(info) return CG.db.char.Pixel[ info[#info] ] end,
			set = function(info, value) CG.db.char.Pixel[ info[#info] ] = value; CG:ActionbarGlow() end,
			args = {
				NumLines = {
					order = 1,
					type = "range",
					name = L["Num Lines"],
					desc = L["Defines the number of lines the glow will spawn."],
					min = 4, max = 16, step = 1,
				},
				Frequency = {
					order = 2,
					type = "range",
					name = L["Frequency"],
					desc = L["Sets the animation speed of the glow. Negative values will rotate the glow anti-clockwise."],
					min = -2, max = 2, step = 0.01,
				},
				Length = {
					order = 3,
					type = "range",
					name = L["Length"],
					desc = L["Defines the length of each individual glow lines."],
					min = 2, max = 16, step = 1,
				},
				Thickness = {
					order = 4,
					type = "range",
					name = L["Thickness"],
					desc = L["Defines the thickness of the glow lines."],
					min = 1, max = 6, step = 1,
				},
				XOffset = {
					order = 5,
					type = "range",
					name = L["X-Offset"],
					min = -5, max = 5, step = 1,
				},
				YOffset = {
					order = 6,
					type = "range",
					name = L["Y-Offset"],
					min = -5, max = 5, step = 1,
				},
			},
		},
		AutoCast = {
			type = "group",
			name = L["AutoCast Glow"],
			order = 22,
			get = function(info) return CG.db.char.AutoCast[ info[#info] ] end,
			set = function(info, value) CG.db.char.AutoCast[ info[#info] ] = value; CG:ActionbarGlow() end,
			args = {
				NumParticles = {
					order = 1,
					type = "range",
					name = L["Num Particles"],
					desc = L["Defines the number of particle groups. Each group contains 4 particles."],
					min = 4, max = 16, step = 1,
				},
				Frequency = {
					order = 2,
					type = "range",
					name = L["Frequency"],
					desc = L["Sets the animation speed of the glow. Negative values will rotate the glow anti-clockwise."],
					min = -2, max = 2, step = 0.01,
				},
				Scale = {
					order = 3,
					type = "range",
					name = L["Scale"],
					min = 0.5, max = 4, step = 0.1,
				},
				XOffset = {
					order = 4,
					type = "range",
					name = L["X-Offset"],
					min = -5, max = 5, step = 1,
				},
				YOffset = {
					order = 5,
					type = "range",
					name = L["Y-Offset"],
					min = -5, max = 5, step = 1,
				},
			},
		},
	},
}

------ [Minimap Button] ------

local LDB = LibStub("LibDataBroker-1.1"):NewDataObject("CoolGlow", {
	type = "data source",
	text = "CoolGlow",
	icon = CG.icon,
	OnClick = function(_, button)
		if button == "LeftButton" then
			CG:ToggleConfig()
		elseif button == "RightButton" then
			CG:ToggleTestFrame()
		end
	end,
	OnTooltipShow = function(tooltip)
		-- build the tooltip
		tooltip:ClearLines()
		tooltip:AddLine(CG.addonName)
		tooltip:AddLine(" ")
		
		-- hints
		--tooltip:AddDoubleLine(L["Left-Click"], L["Open Config"], 1, 1, 1, 0.95, 0.95, 0.32)
		--tooltip:AddDoubleLine(L["Right-Click"], L["Display Glow"], 1, 1, 1, 0.95, 0.95, 0.32)
		tooltip:AddLine(L["|cffffffffLeft-Click|r to open the option settings."], 0.95, 0.95, 0.32)
		tooltip:AddLine(L["|cffffffffRight-Click|r to display the current proc glow."], 0.95, 0.95, 0.32)
		
		tooltip:Show()		
	end,
})

---- [Core Framework] ----

function CG:OnInitialize()
	CG.db = LibStub("AceDB-3.0"):New("CoolGlowDB", defaults)
	LibStub("AceConfig-3.0"):RegisterOptionsTable("Cool Glow", options)
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Cool Glow")
	LDBI:Register("CoolGlow", LDB, CG.db.char.minimap)
end

function CG:OnEnable()
    self:ActionbarGlow()
	self:CreateTestFrame()
	
	self:RegisterChatCommand("cg", "ToggleConfig")
    self:RegisterChatCommand("coolglow", "ToggleConfig")
	self:RegisterChatCommand("cgt", "ToggleTestFrame")
	self:RegisterChatCommand("cgtest", "ToggleTestFrame")
end

function CG:ToggleConfig()
	if InterfaceOptionsFrame:IsShown() then
		InterfaceOptionsFrame:Hide()
	else
		InterfaceOptionsFrame:Show()
		InterfaceOptionsFrame_OpenToCategory("Cool Glow")
	end
end

local SendMessageWaiting -- only allow 1 delay at a time regardless of eventing
function CG:SendMessage()
	if IsInRaid() then
		C_ChatInfo.SendAddonMessage("COOLGLOW_VERSIONCHECK", CG.version, (not IsInRaid(LE_PARTY_CATEGORY_HOME) and IsInRaid(LE_PARTY_CATEGORY_INSTANCE)) and "INSTANCE_CHAT" or "RAID")
	elseif IsInGroup() then
		C_ChatInfo.SendAddonMessage("COOLGLOW_VERSIONCHECK", CG.version, (not IsInGroup(LE_PARTY_CATEGORY_HOME) and IsInGroup(LE_PARTY_CATEGORY_INSTANCE)) and "INSTANCE_CHAT" or "PARTY")
	end

	SendMessageWaiting = nil
end

function CG:CHAT_MSG_ADDON(event, prefix, message, channel, sender)
	if not CG.db.char.Notify then return end
	if sender == UnitName("player") then return end
	
	if prefix == "COOLGLOW_VERSIONCHECK" then
		local msg, ver = tonumber(message), tonumber(CG.version)
		if msg and (msg > ver) then -- you're outdated!
			if not CG.recievedOutOfDateMessage then
				CG:Print((L["Your version of %s is out of date. Latest version is |cfff2f251%d|r."]):format(CG.addonName, message))
				
				if msg and ((msg - ver) >= 0.05) then
					StaticPopup_Show("COOLGLOW_UPDATE_AVAILABLE")
				end

				CG.receivedOutOfDateMessage = true -- to prevent receiving this message more than once
			end
		elseif msg and (msg < ver) then -- Send Message Back
			if not SendMessageWaiting then
				SendMessageWaiting = CG:ScheduleTimer(CG.SendMessage, 10)
			end
		end
	end
end
CG:RegisterEvent("CHAT_MSG_ADDON")
C_ChatInfo.RegisterAddonMessagePrefix("COOLGLOW_VERSIONCHECK")

local SendRecieveGroupSize = 0
function CG:GROUP_ROSTER_UPDATE()
	local num = GetNumGroupMembers()
	if num ~= SendRecieveGroupSize then
		if num > 1 and num > SendRecieveGroupSize then
			if not SendMessageWaiting then
				SendMessageWaiting = CG:ScheduleTimer(CG.SendMessage, 10)
			end
		end
		SendRecieveGroupSize = num
	end
end
CG:RegisterEvent("GROUP_ROSTER_UPDATE")

function CG:PLAYER_ENTERING_WORLD()
	if CG.db.char.Welcome then
		CG:Print((L["Version |cfff2f251%s|r is loaded. Thank you for using it."]):format(CG.version))
	end
	
	if not SendMessageWaiting then
		SendMessageWaiting = CG:ScheduleTimer(CG.SendMessage, 10)
	end
	CG:UnregisterEvent("PLAYER_ENTERING_WORLD")
end
CG:RegisterEvent("PLAYER_ENTERING_WORLD")

function CG:CreateTestFrame()
	local test = CreateFrame("Frame", "CoolGlowTestFrame", UIParent)
	test:ClearAllPoints()
	test:SetPoint("TOP", UIParent, "TOP", 0, -110)
	test:SetSize(50, 50)
	if IsAddOnLoaded("ElvUI") then
		test:SetTemplate("Transparent")
	else
		test:SetBackdrop({
			bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
			edgeFile = "edgeFile", tile = false, tileSize = 0, edgeSize = 1,
			insets = {top = 0, left = 0, bottom = 0, right = 0},
		})
		test:SetBackdropColor(18/255, 18/255, 18/255, 0.6)
		test:SetBackdropBorderColor(0, 0, 0, 1)
	end
	test:SetFrameStrata("HIGH")
	test:Hide()
	
	test:SetMovable(true)
	test:SetResizable(true)
	test:EnableMouse(true)
	test:SetClampedToScreen(true)
	test:RegisterForDrag("LeftButton")
	test:SetMinResize(10, 10)
	test:SetMaxResize(100, 100)
	test:SetScript("OnMouseDown", function(self, button) 
		if button == "RightButton" and not self.isSizing then
			self:StartSizing()
			self.isSizing = true
		end
	end)
	test:SetScript("OnMouseUp", function(self, button) 
		if button == "RightButton" and self.isSizing then
			self:StopMovingOrSizing()
			self.isSizing = false
			CGSize.Width = self:GetWidth()
			CGSize.Height = self:GetHeight()
		end
	end)
	test:SetScript("OnDragStart", test.StartMoving)
	test:SetScript("OnDragStop", function(self)
	  self:StopMovingOrSizing()
	  CGPosition.XPos = self:GetLeft()
	  CGPosition.YPos = self:GetBottom()
	end)
end

function CG:PLAYER_LOGIN()
	CoolGlowPerCharDB = CoolGlowPerCharDB or {} -- create table if one doesn't exist
	CGPosition = CoolGlowPerCharDB -- assign settings declared above
	CGSize = CoolGlowPerCharDB -- assign settings declared above
	
	if CGPosition.XPos then
		CoolGlowTestFrame:ClearAllPoints()
		CoolGlowTestFrame:SetPoint("BOTTOMLEFT", CGPosition.XPos, CGPosition.YPos)
	end
	
	if CGSize.Width then
		CoolGlowTestFrame:SetSize(CGSize.Width, CGSize.Height)
	end
end
CG:RegisterEvent("PLAYER_LOGIN")

function CG:ToggleTestFrame()
    if CoolGlowTestFrame:IsShown() then
		CG:StopGlow(CoolGlowTestFrame)
		CoolGlowTestFrame:Hide()
    else
		CG:StartGlow(CoolGlowTestFrame)
		CoolGlowTestFrame:Show()
    end
end

---- [Core Function] ----

function CG:StartGlow(button)
	local Color = {CG.db.char.Color.r, CG.db.char.Color.g, CG.db.char.Color.b, CG.db.char.Color.a or 1}
	if CG.db.char.Style == "Button" then
		LCG.ButtonGlow_Start(button, Color, CG.db.char.Button.Frequency)
	elseif CG.db.char.Style == "Pixel" then
		LCG.PixelGlow_Start(button, Color, CG.db.char.Pixel.NumLines, CG.db.char.Pixel.Frequency, CG.db.char.Pixel.Length, CG.db.char.Pixel.Thickness, CG.db.char.Pixel.XOffset, CG.db.char.Pixel.YOffset)
	elseif CG.db.char.Style == "AutoCast" then
		LCG.AutoCastGlow_Start(button, Color, CG.db.char.AutoCast.NumParticles, CG.db.char.AutoCast.Frequency, CG.db.char.AutoCast.Scale, CG.db.char.AutoCast.XOffset, CG.db.char.AutoCast.YOffset)
	end
end

function CG:StopGlow(button)
	LCG.ButtonGlow_Stop(button)
	LCG.PixelGlow_Stop(button)
	LCG.AutoCastGlow_Stop(button)
end

function CG:ActionbarGlow()
	if IsAddOnLoaded("ElvUI") or IsAddOnLoaded("Bartender4") then
		C_Timer.After(0,function()
			if LibStub then
				local lib = LibStub:GetLibrary("LibButtonGlow-1.0",4)
				if lib then
					function lib.ShowOverlayGlow(button)
						if button:GetAttribute("type") == "action" then
							local actionType,actionID = GetActionInfo(button:GetAttribute("action"))
							CG:StartGlow(button)
						end
						function lib.HideOverlayGlow(button)
							CG:StopGlow(button)
						end
					end
				end
			end
		end)
	else
		hooksecurefunc("ActionButton_ShowOverlayGlow", function(self)
			ActionButton_OverlayGlowAnimOutFinished(self.overlay.animOut)
			CG:StartGlow(self)
		end)
		
		hooksecurefunc("ActionButton_HideOverlayGlow", function(self)
			CG:StopGlow(self)
		end)
	end
end

------ [Static Popup] ------

StaticPopupDialogs["COOLGLOW_RLUI"] = {
	text = L["One or more of the changes you have made require you to reload your UI."],
	button1 = ACCEPT,
	button2 = CANCEL,
	OnAccept = function() ReloadUI() end,
	timeout = 0,
	whileDead = 1,
	hideOnEscape = false,
}

StaticPopupDialogs['COOLGLOW_UPDATE_AVAILABLE'] = {
	text = L["Cool Glow is five or more revisions out of date. You can download the newest version from www.curseforge.com!"],
	hasEditBox = 1,
	OnShow = function(self)
		self.editBox:SetAutoFocus(false)
		self.editBox.width = self.editBox:GetWidth()
		self.editBox:Width(220)
		self.editBox:SetText("https://www.curseforge.com/wow/addons/cool-glow")
		self.editBox:HighlightText()
		ChatEdit_FocusActiveWindow();
	end,
	OnHide = function(self)
		self.editBox:Width(self.editBox.width or 50)
		self.editBox.width = nil
	end,
	hideOnEscape = 1,
	button1 = OKAY,
	OnAccept = function() end,
	EditBoxOnEnterPressed = function(self)
		ChatEdit_FocusActiveWindow();
		self:GetParent():Hide();
	end,
	EditBoxOnEscapePressed = function(self)
		ChatEdit_FocusActiveWindow();
		self:GetParent():Hide();
	end,
	EditBoxOnTextChanged = function(self)
		if(self:GetText() ~= "https://www.curseforge.com/wow/addons/cool-glow") then
			self:SetText("https://www.curseforge.com/wow/addons/cool-glow")
		end
		self:HighlightText()
		self:ClearFocus()
		ChatEdit_FocusActiveWindow();
	end,
	OnEditFocusGained = function(self)
		self:HighlightText()
	end,
	showAlert = 1,
}